=begin

=NAME
 ga.rbx - Gene Structural Attributes Graphics (PrettyGraphics)

=USAGE
 <img src='ga.rbx?name=rho&len=234&gn=b0001&gp=118..234&gw=15&gs=d'>

  CGI arguments
    name=Strings name    (unique required)
    len=Int      length  (unique required)
    +gn=Strings  gene name         (required)
    +gp=n..m     gene position     (required) 
    +gw=Int      gene object width (required-) 
    +gs=[dc]     strand of gene    (required) 
    +fs=Int      fontsize of gene label (required-)

   gw and fs is able to 

=AUTHOR
 Mitsuteru S. Nakao <n@BioRuby.org>

=LICENCE
 GPL.

=end


require 'libplot'
require 'cgi'

fmt='png'
tmpdir='/tmp/WWW/hoge'
file_name="#{tmpdir}/g#{$$}.#{fmt}"
font='HersheySans'
http_header="HTTP/1.0 200 OK\n"+"Content-Type: image/#{fmt}\n\n"
STDOUT.print http_header

# CGI arguments
c=CGI.new

# set scaling factor (image width)
if c['sf'][0];  sf=1.0 / c['sf'][0].to_f
else;           sf=1;        end
#
if c['locus'][0]   
  l5,l3=c['locus'][0].split(/\.\./);  l5=l5.to_i;  l3=l3.to_i
  len=l3 - l5
  if len * sf > 100000  # safe guard
    name="Too long query (#{len} aa) (--;"
    len=200 
  end
else          
  len=134 # default
end

# set sequence line width(sw)
if c['sw'][0];  sw=c['sw'][0].to_i;  else;           sw=3;  end

# image width (h) and height (h)
w=( sf * len + (l5.to_s + l3.to_s).length * 5 + 20 ).to_i 
h=41


# set up to draw
# set color
bgcolor='white'
basecolor='gray'
outlinecolor='gray87'  
poslabelcolor='gray55'
labelcolor='gray98'
labelshadowcolor='gray60'

f=File.open(file_name, "w+")
p=Libplot::PlotterParams.new()
p.setplparam("BITMAPSIZE", "#{w}x#{h}")
p.setplparam("BG_COLOR", bgcolor)
i=Libplot::Plotter.new(fmt, STDIN, f, STDERR, p)


# drawing setup
i.openpl();    i.space(0,0, w,h)


# draw outline
unless c['nool'].is_a?(Array) and c['nool'][0]
  i.pencolorname(outlinecolor);  i.box(0,0, w,h)   
end


# draw 5' and 3' end position labels
shift=l5.to_s.length * 5 + 10
i.pencolorname(poslabelcolor);  i.fontname(font);  i.fontsize(10); 
i.move(4, h / 2);                           i.alabel("l", "c", "#{l5}")
i.move(shift + sf * len + 1, h / 2);        i.alabel("l", "c", "#{l3}")


# draw genes/orfs and elements (as gp=n..n+1)
# foreach gn, gp, gc, gw
if c['gn'].is_a?(Array) and c['gp'].is_a?(Array) and c['gc'].is_a?(Array) 
  0.upto(c['gn'].length - 1) do |nnn|
    s,e=c['gp'][nnn].split(/\.\./);     s=s.to_i;   e=e.to_i
    if c['gs'] and c['gs'][nnn]; strand=c['gs'][nnn];    
    else;  strand="d";  end
    if c['gw'] and c['gw'][nnn]; width=c['gw'][nnn].to_i;  
    else;  width="11";  end
    if c['fs'] and c['fs'][nnn]; fs=c['fs'][nnn].to_i;     
    else;  fs=9;        end
    if strand == "d";      y=h / 2 + width / 2     # set y coord.
    elsif strand == "c";   y=h / 2 - width / 2
    else;                  y=h / 2;                end
    # draw Orf line
    i.pencolorname(c['gc'][nnn])
    i.linewidth(width)
    i.line(sf * (s - l5) + shift, y, sf * (e - l5) + shift, y)  
    # write orf name with shadow
    i.fontname(font);   i.fontsize(12)
    i.move(sf * (s - l5 + (e - s) / 2) + shift + 1, y - 1)
    i.pencolorname(labelshadowcolor);     i.alabel("c", "c", c['gn'][nnn])
    # write orf name 
    i.move(sf * (s - l5 + (e - s) / 2) + shift, y )
    i.pencolorname(labelcolor);           i.alabel("c", "c", c['gn'][nnn])
  end
else
end


# draw base line
i.pencolorname(basecolor);   i.linewidth(sw);  
i.line(shift, h / 2, shift + sf * len, h / 2)

# finish
i.closepl()
f.close

# ? 
f=File.open(file_name)
while l=f.gets
  print l
end
f.close

# 
File.delete(file_name)
