=begin

=NAME
 pa.rb - Protein Attributes Graphics (PrettyGraphics)

=USAGE
 <img src='pa.rbx?name=rho&len=234&mn=ATP&mm125..156&mc=red2'>

  CGI arguments
    name=Strings name    (unique required)
    len=Int      length  (unique required)
    +mn=Strings  motif name         (required)
    +mm=n..m     motif position     (required) 
    +mw=Int      motif object width (required-) 
    +mc=color    color of gene      (required) 

  color is a Libplotcolorname (cf. /usr/local/share/libplot/colors.txt).
  

=AUTHOR
 Mitsuteru S. Nakao <n@BioRuby.org>

=LICENCE
 GPL.

=end

require 'libplot'
require 'cgi'

fmt='png'
tmpdir='/tmp/WWW/hoge'
file_name="#{tmpdir}/p#{$$}.#{fmt}"

c=CGI.new

if c['name'][0]; name=c['name'][0];   else;      name='hoge';   end
# sequence length(len)
if c['len'][0]
  len=c['len'][0].to_i
  if len > 10000  # safe guard
    name="Too long query (#{len} aa) (--;"
    len=200
  end
else          
  len=234 # default
end
if c['sw'][0];  sw=c['sw'][0].to_i;   else;             sw=3;   end

# set image width(w) and height(h)
w=len + name.length * 5 + 20
h=21

# send HTTP header
http_header="HTTP/1.0 200 OK\n"+"Content-Type: image/#{fmt}\n\n"
STDOUT.print http_header 

# drawing setup
bgcolor='white'
outlinecolor='gray86'
proteinnamecolor='black'
proteinseqcolor='gray'
motifnamecolor='gray95'

f=File.open(file_name, "w+")
p=Libplot::PlotterParams.new()
p.setplparam("BITMAPSIZE", "#{w}x#{h}")
p.setplparam("BG_COLOR", bgcolor)
i=Libplot::Plotter.new(fmt, STDIN, f, STDERR, p)

i.openpl()
i.space(0, 0, w, h)

# draw outline
i.pencolorname(outlinecolor);      i.box(0,0, w, h)   # outline

# write protein name
shift=5
i.move(shift + len + 2, h / 2);
i.pencolorname(proteinnamecolor);  i.fontname("HersheySans");  i.fontsize(10)
i.alabel("l", "c", "#{name}")

# draw protein sequence
i.pencolorname(proteinseqcolor);   i.linewidth(sw)
i.line(shift,h / 2, shift + len,h / 2)

# draw motifs
if c['mn'].is_a?(Array) and c['mm'].is_a?(Array) and c['mc'].is_a?(Array) 
  i.fontsize(9)
  # foreach mn,mw,mm,mw,fs
  0.upto(c['mn'].length - 1) do |nnn|
    i.pencolorname(c['mc'][nnn])
    s,e=c['mm'][nnn].split(/\.\./);   s=s.to_i;   e=e.to_i
    if c['mw'] and c['mw'][nnn]; width=c['mw'][nnn].to_i; else; width=11; end
    if c['fs'] and c['fs'][nnn]; fs=c['fs'][nnn].to_i;    else;     fs=9; end
    # draw motif/domain object
    i.linewidth(width)
    i.line(s + shift,h / 2, e + shift,h / 2)
    # write motif/domain name
    i.move(s + shift + (e - s) / 2, h / 2 )
    i.pencolorname(motifnamecolor);    
    i.alabel("c", "c", c['mn'][nnn])
  end
else
end

i.closepl()
f.close

# 
f=File.open(file_name)
while l=f.gets
  print l
end
f.close

#
File.delete(file_name)
